@@ -1,13 +1,23 @@
 import sys
-import shutil
 import argparse
-import platformdirs
 import configparser
 from pathlib import Path
 import subprocess as sub
+from os import environ as env
 from argparse import RawDescriptionHelpFormatter
 
 
+def get_version():
+	try:
+		with open(resource_path('VERSION')) as f:
+			version = f.read()
+	except FileNotFoundError:
+		# this should never happen in a built application
+		# only if there is no VERSION file while developing locally
+		version = 'n/a'
+	return version
+
+
 class SSOSessionError(Exception):
 	pass
 
@@ -23,23 +33,15 @@ def resource_path(relative_path):
 
 
 def load_config():
-	config_dir = Path(platformdirs.user_config_dir('klogin'))
-	profiles_path = config_dir.joinpath('profiles')
+	aws_config = Path('~/.aws/config').expanduser()
 	config = configparser.ConfigParser()
-	print(f'loading config from {profiles_path}')
-	#p = Path(config_path).expanduser()
-	#profiles_path = resource_path('profiles')
-	if not profiles_path.exists():
-		print('no config found - using default profiles')
-		return None
-	config.read(profiles_path)
+	config.read(aws_config)
 	profiles = {}
 	for section in config.sections():
 		_section = section.split('profile ')[-1]
 		profiles[_section] = {}
 		for option in config.options(section):
 			profiles[_section][option] = config.get(section, option)
-	#print(f"config loaded: {profiles}")
 	return profiles
 
 
@@ -57,25 +59,20 @@ def write_profiles(kconfig):
 		awsconfig.write(f)
 
 
-def execute(cmd):
-	config_dir = Path(platformdirs.user_config_dir('klogin'))
-	profiles_path = config_dir.joinpath('profiles')
-	#profiles_path = resource_path('profiles')
-	print(f'executing command <{cmd}> with AWS_CONFIG_FILE={profiles_path}')
-	sub.run(
+def execute(cmd, capture=True):
+	print(f"executing command: {cmd}")
+	return sub.run(
 		cmd,
 		shell=True,
 		check=True,
-		capture_output=True,
-		env={
-			'AWS_CONFIG_FILE': profiles_path
-		}
+		capture_output=capture,
+		env=env
 	)
 
 
 def sso_login(profile):
 	command = f'aws sso login --profile {profile}'
-	execute(command)
+	execute(command, capture=False)
 
 
 def update_kubeconfig(cluster_name, alias, profile):
@@ -86,10 +83,16 @@ def update_kubeconfig(cluster_name, alias, profile):
 		f'--alias {alias}'
 	)
 	try:
-		execute(command)
+		p = execute(command)
+		print(p.stdout.strip().decode())
 	except sub.CalledProcessError as ex:
 		msg = ex.stderr.strip().decode()
-		if msg.startswith('Error loading SSO Token:'):
+		handled_errors = (
+			'Error loading SSO Token:',
+			'The SSO session associated'
+		)
+		# check if the error message startswith any of the known errors
+		if msg.startswith(handled_errors):
 			raise SSOSessionError(msg)
 		else:
 			print(ex.stderr.decode())
@@ -119,16 +122,77 @@ def login(cluster_name, cluster_alias, profile, sso_region):
 		update_kubeconfig(cluster_name, cluster_alias, profile)
 	except SSOSessionError:
 		sso_login(f'sso-{sso_region}')
+		update_kubeconfig(cluster_name, cluster_alias, profile)
 
 
-def bootstrap():
+def print_section(section, items):
+	print(f"[{section}]")
+	for key, value in items:
+		print(f'{key} = {value}')
+
+
+def print_config(config):
+	for section in config.sections():
+		print_section(section, config.items(section))
+		print()
+
+
+def confirm(msg, skip=False):
+	if skip:
+		return True
+	answer = input(msg)
+	return answer.lower() in ['y', 'yes']
+
+
+def bootstrap(dry=False, skip_confirm=False):
 	print('bootstrap')
-	config_dir = Path(platformdirs.user_config_dir('klogin'))
-	if not config_dir.exists():
-		print('config_dir does not exist - creating it')
-		config_dir.mkdir(parents=True)
-	profiles_path = config_dir.joinpath('profiles')
-	shutil.copy(resource_path('profiles'), profiles_path)
+	aws_config_dir = Path('~/.aws/').expanduser()
+	aws_config = Path('~/.aws/config').expanduser()
+	current_config = configparser.ConfigParser(allow_no_value=True)
+	if aws_config.exists():
+		current_config.read(aws_config)
+	else:
+		aws_config_dir.mkdir(parents=True)
+		aws_config.touch()
+	profiles_path = resource_path('profiles')
+	baked_config = configparser.ConfigParser()
+	baked_config.read(profiles_path)
+	for section in baked_config.sections():
+		if current_config.has_section(section):
+			print_section(section, baked_config.items(section))
+			print()
+			print_section(section, current_config.items(section))
+			print()
+			if confirm(f"Overwrite '{section}' [y/n] ? ", skip=skip_confirm):
+				try:
+					current_config.add_section(section)
+				except configparser.DuplicateSectionError:
+					pass
+				for key, value in baked_config.items(section):
+					current_config.set(section, key, value)
+				print(f"section {section} added")
+			print()
+		else:
+			print_section(section, baked_config.items(section))
+			print()
+			if confirm(f"Add '{section}' [y/n] ? ", skip=skip_confirm):
+				try:
+					current_config.add_section(section)
+				except configparser.DuplicateSectionError:
+					pass
+				for key, value in baked_config.items(section):
+					current_config.set(section, key, value)
+				print(f"section {section} added")
+			print()
+	print('----------------------------------------------------------')
+	print_config(current_config)
+	print('----------------------------------------------------------')
+	if not dry:
+		if confirm(f"Write config to {aws_config} [y/n] ? ", skip=skip_confirm):
+			with open(str(aws_config), 'w') as f:
+				current_config.write(f)
+			print(f"config written to {aws_config}")
+
 	print('bootstrap complete')
 
 
@@ -136,10 +200,17 @@ def main():
 	parser = argparse.ArgumentParser(
 		prog = 'klogin',
 		description = 'Log in to GenuityScience kubernetes clusters',
-		epilog = 'Example usage:\nklogin -c=/home/user/myklogin.yml platform-dev admin',
+		epilog = 'Example usage:\nklogin platform-dev admin',
 		formatter_class=RawDescriptionHelpFormatter
 	)
-	#parser.add_argument('-c', '--config', nargs='?', default='~/klogin.yml')
+	parser.add_argument(
+		'-v',
+		'--version',
+		default=False,
+		action='store_true',
+		dest='show_version',
+		help='Show version'
+	)
 	cluster = parser.add_argument_group(
 		'cluster',
 		'''
@@ -156,15 +227,32 @@ def main():
 		Example: klogin profile add
 		'''
 	)
+	profiles.add_argument(
+		'-d',
+		'--dry',
+		nargs='?',
+		default=False,
+		help='Dry run initialization'
+	)
+	profiles.add_argument(
+		'-y',
+		'--yes',
+		default=False,
+		dest='skip_confirm',
+		action='store_true',
+		help='Skip confirmation and add/overwrite all profiles on init'
+	)
 	parser.add_argument("second", nargs='?', help='<role>/<command>')
 	args = parser.parse_args()
-	print(f"args: {args}")
 	# if no arguments are provided we exit and show the help message
 	if len(sys.argv) == 1:
 		parser.print_help(sys.stderr)
 		sys.exit(1)
-	if args.first in ['bootstrap', 'init']:
-		bootstrap()
+	if args.show_version:
+		print(f'version: {get_version()}')
+		sys.exit(0)
+	if args.first in ['init']:
+		bootstrap(args.dry, args.skip_confirm)
 		sys.exit(0)
 	config = load_config()
 	if args.first in ['profile', 'profiles']:
@@ -172,11 +260,10 @@ def main():
 	else:
 		cluster_alias = args.first
 		role = args.second or 'ro'
-		# construct the profile name, <clustername>-<rolename>
+		# construct the profile name, <cluster_alias>-<rolename>
 		# for cluster platform-dev and role admin: platform-dev-admin 
 		# for cluster gor-dev and role ro: gor-dev-ro
 		profile = f'{cluster_alias}-{role}'
-		#cluster_name = config['clusters'][cluster_alias]['name']
 		cluster_name = config[profile]['cluster_name']
 		sso_region = config[profile]['sso_region']
 		login(cluster_name, cluster_alias, profile, sso_region)
